/**
 *Include file s817.h
 *
 *Types and constants.
 *This file is visible to applications.
 *
 *Copyright (C) Sensoray Co., Inc. 2007-2008
 **/

#ifndef _S817_H_
#define _S817_H_

typedef int HANDLE;

#define SYS_BOARDS 4uL          /* max boards */
#define SYS_OUT_PER_BOARD       4  /*outputs per board */

typedef struct {
    int             boards;                 /*number of detected boards*/
    HANDLE          hBrdEvent[SYS_BOARDS];  /*board event handles*/
} SYSTEM;


/*Capture buffer struct: contains pointers to image buffers */
typedef struct {
    int     bufId;          /*buffer index*/
    char    *pData;         /*image data*/
    char    *pStat;   /* status data */
    int width;
    int height;
} CAP_BUFFER;

/*Operating mode consts and struct*/
/*Image size (JPEG)*/
typedef enum {
    SIZE_4CIF,              /*640x480 (NTSC)*/
    SIZE_2CIF,              /*640x240 (NTSC)*/
    SIZE_1CIF,              /*320x240 (NTSC)*/
    SIZE_4CIFI,             /*640x480 (NTSC) interpolated*/
    SIZE_JPG_MAX            /*dummy for range checking*/
} MODE_JPG_SIZE;

/*line sizes in pixels*/
#define LINE_NTSC_FULL          640
#define LINE_PAL_FULL           704

#define LINES_PER_FIELD_NTSC    240
#define LINES_PER_FIELD_PAL     288

/*TV standard*/
typedef enum {
    TV_NTSC,
    TV_PAL,
    TV_SECAM,
    TV_MAX                          /*dummy for range checking*/
} MODE_TV_FORMAT;

typedef enum {
    CAP_MODE_CONT,
    CAP_MODE_SINGLE,
    CAP_MODE_MAX                    /*dummy for range checking*/
} CAP_MODE;

#define CAP_OFF         0                               /*capture off*/
#define CAP_JPG         0x1                             /*capture jpegs*/
#define CAP_BMP         0x2                             /*capture bitmaps*/

typedef enum {
	BMP_Y8,
	BMP_MAX
} MODE_BMP_TYPE;

typedef struct {
    MODE_TV_FORMAT  tvFormat;
    MODE_JPG_SIZE   jpgSize;
    MODE_BMP_TYPE   bmpType;
    int                             brightness;             /*brightness*/
    int                             contrast;               /*contrast*/
    int                             saturation;             /*saturation*/
    int                             hue;                    /*hue*/
} VIDEO_MODE;

typedef struct {
    /*capture mode (single/continuous)*/
    CAP_MODE        capMode; 
    /*capture types: combination of CAP_JPG,
      CAP_BMP, both, neither*/
    int             capType; 
    /*compression quality (COMP_QUAL_MIN..COMP_QUAL_MAX)*/
    int                     compQuality;
    /*frame rate (decimation const) for BMP capture*/
    int                     frameRateBmp;
    /*frame rate (decimation const) for JPG capture*/
    int                     frameRateJpg;
} CAPTURE_MODE;
 
typedef struct {
    VIDEO_MODE              vMode;
    CAPTURE_MODE    cMode;
} MODE;

#define COMP_QUAL_MIN                   0
#define COMP_QUAL_MAX                   100

/*capture frame rates (decimation constants)*/
/*Must contain no more than 16 (dummy excluded) entries!*/
enum {
    FR_FULL,                /*full: 30(NTSC)/25(PAL)*/
    FR_4_5,                 /*4/5:  24/20*/
    FR_2_3,                 /*2/3:  20/16.7*/
    FR_1_2,                 /*1/2:  15/12.5*/
    FR_2_5,                 /*2/5:  12/10*/
    FR_1_3,                 /*1/3:  10/8.3*/
    FR_1_5,                 /*1/5:  6/5*/
    FR_1_10,                /*1/10: 3/2.5*/
    FR_1_15,                /*1/15: 2/1.7*/
    FR_MAX                  /*dummy for range checking*/
};

#define DEFAULT_VIDEO_MODE      {TV_NTSC, SIZE_4CIF, BMP_Y8, 0x80, 0x80, 0x80, 0}

#define DEFAULT_CAPTURE_MODE    {CAP_MODE_CONT, CAP_OFF, 60, FR_FULL, FR_FULL}

#define DEFAULT_MODE            {DEFAULT_VIDEO_MODE, DEFAULT_CAPTURE_MODE}

#define DEFAULT_OSD_MODE            {1, 0, 0, 0, 0, 0, 0, "" }


typedef struct {
    /*OSD on if != 0*/
    int                             osdOn;
    /*OSD on bitmap if !=0*/
    int                             osdBmp;                         
    /*transparent OSD if !=0 */
    int                             transparent;            
    /*OSD string on top of the screen  if !=0 */
    int                             positionTop;            
    /*date format is dd-mm if != 0*/
    int                             ddmm;
    /*year value in date is truncated to 2 digits if != 0*/
    int                             year2;
    /*number of digits showing fraction of a second: 0, 1, 2*/
    int                             fraction;
    /*caption text*/
    char                            line[80];
} OSD_MODE;


/*capture status data */
typedef struct {
    int board;              //board index (inserted by host)
    int chanId;             //channel index (1..16)
    int bufId;              //buffer index
    int capType;            //capture type present in buffer: CAP_JPG or CAP_BMP
    int videopresent;       //video present flag from decoder
    int valid;              //=0 if jpg size > max allowed (if applies)
    int jpgsize;            //size of jpg data (if applies)
    unsigned int tick;      //DSP tick (1ms)
    int frameCnt;           //captured frames counter value per capture type per channel
    int reserved[4];        //debug
} CAP_STAT;

/*Extended error reporting*/
typedef struct {
    unsigned int                    tick;           /*1 ms timer tick*/
    unsigned int                    ecode;          /*error code*/
    unsigned int                    reserved[2];    /**/
} EXTERR;

#define SYS_EXTERR_BUF_SIZE     64 /*number of EXTERR elements in ring buffer*/

#define START_ERR_CODE 200

typedef int ECODE;
/*Error codes*/
#define ERR_SX_OPEN  (START_ERR_CODE+1)          /*unable to open the driver*/
#define ERR_BRIDGE_NOT_FOUND (START_ERR_CODE+2) /*no 817 bridge found*/
#define ERR_SELECT_DEVICE (START_ERR_CODE+3)    /*selected board not found*/
#define ERR_MEM_ALLOC (START_ERR_CODE+4)        /*memory allocation error*/
#define ERR_MEM_LOCK (START_ERR_CODE+5)         /*DMA buffer lock error*/
#define ERR_I2CINIT_BUSY (START_ERR_CODE+6)     /*I2C bus initialization error*/
#define ERR_I2CINIT_ERR (START_ERR_CODE+7)      /*I2C bus initialization error*/
#define ERR_TIMEOUT (START_ERR_CODE+8)          /*internal access timeout*/
#define ERR_LOADING_DLL (START_ERR_CODE+9)      /*error loading DLL*/
#define ERR_FUNCADDR (START_ERR_CODE+10)        /*error getting a DLL func*/
#define ERR_DLL (START_ERR_CODE+11)                /*error accessing DLL*/
#define ERR_NO_REGISTERED_DEV (START_ERR_CODE+12)   /*unable to register boards*/
#define ERR_I2C (START_ERR_CODE+13)                /*internal I2C bus error*/
#define ERR_I2CTO (START_ERR_CODE+14)              /*internal I2C timeout*/
#define ERR_EEPROM (START_ERR_CODE+15)             /*EEPROM programming error*/
#define ERR_XPTO (START_ERR_CODE+16)     /*internal crosspoint switch timeout*/
#define ERR_ILLEGAL_MODE (START_ERR_CODE+17) /*illegal value in MODE structure*/
#define ERR_INTERNAL_OBJ (START_ERR_CODE+18) /*internal obj allocation error*/
#define ERR_ACQ_TIMEOUT (START_ERR_CODE+19)  /*acquisition timeout*/
#define ERR_INTERRUPT (START_ERR_CODE+20)    /*interrupt init error*/
#define ERR_ILLEGAL_INDEX (START_ERR_CODE+23)      /*illegal board index*/
#define ERR_PCI (START_ERR_CODE+24)                /*PCI access error*/
#define ERR_BOARDADDR (START_ERR_CODE+25)    /*board address mapping failed*/
#define ERR_BOARDREG (START_ERR_CODE+26)     /*error registering the board*/
#define ERR_ITEMS (START_ERR_CODE+27)        /*board has no items*/
#define ERR_ALREADYOPEN (START_ERR_CODE+28)  /*board already open*/
#define ERR_FIFOEMPTY (START_ERR_CODE+29)    /*data FIFO empty*/
/*internal process data access timeout*/
#define ERR_DATA_ACCESS_TO (START_ERR_CODE+30)
#define ERR_DEVICEINFO (START_ERR_CODE+31)   /*driver error*/
#define ERR_DSP_INIT (START_ERR_CODE+34)   /*less than expected DSPs on PCI*/
#define ERR_BOARD_NOT_FOUND (START_ERR_CODE+35) 
#define ERR_ILLEGAL_BOARD (START_ERR_CODE+36)      /*illegal board index*/
#define ERR_ILLEGAL_CHAN (START_ERR_CODE+37)       /*illegal channel #*/
#define ERR_NO_FREE_BUFFER (START_ERR_CODE+38)    /*no free buffers*/
#define ERR_DSP_START_TO (START_ERR_CODE+39)      /*DSP start timeout*/
/*HOST-to-DSP command response timeout 40*/
#define ERR_DSP_CMD_TO (START_ERR_CODE+40)  
/*HOST-to-DSP command send error (DSP busy)*/
#define ERR_DSP_CMD_BUSY (START_ERR_CODE+41)
/*DSP timeout while receiving host data*/
#define ERR_DSP_DATA_TO (START_ERR_CODE+42)
/*DSP parse error*/
#define ERR_DSP_PARSE_DATA (START_ERR_CODE+43)
/*error allocating FIFO*/
#define ERR_ALLOC_FIFO (START_ERR_CODE+44)
/*FIFO contains no data*/
#define ERR_FIFO_EMPTY (START_ERR_CODE+45)
/*invalid address passed to DSP*/
#define ERR_DSP_INVALID_ADDR (START_ERR_CODE+46)
#define ERR_DSP_DATA_TO_CAP (START_ERR_CODE+47)
/*channel already capturing (StartCapture attempted)*/
#define ERR_ALREADY_CAPTURING (START_ERR_CODE+48)
/*channel not capturing (StopCapture attempted)*/
#define ERR_NOT_CAPTURING (START_ERR_CODE+49)     
/*illegal compression quality value 50*/
#define ERR_ILLEGAL_COMPQUALITY (START_ERR_CODE+50)
#define ERR_ILLEGAL_FRAMERATE (START_ERR_CODE+51)  /*illegal frame rate*/
#define ERR_ILLEGAL_TVFORMAT (START_ERR_CODE+52)   /*illegal TV format*/
#define ERR_ILLEGAL_JPGSIZE (START_ERR_CODE+53)    /*illegal jpeg size*/
/*error opening firmware file*/
#define ERR_FW_FILE_OPEN (START_ERR_CODE+54)
/*error reading firmware file*/
#define ERR_FW_FILE_READ (START_ERR_CODE+55)
/*illegal dsp index*/
#define ERR_ILLEGAL_DSP_INDEX (START_ERR_CODE+56)
/*illegal buffer index*/
#define ERR_ILLEGAL_BUF (START_ERR_CODE+57)
/*DSP address not configured on host*/
#define ERR_ADDR_NOT_CONFIG (START_ERR_CODE+58)
/*illegal capture mode*/
#define ERR_ILLEGAL_CAPMODE (START_ERR_CODE+59)
#define ERR_ILLEGAL_BUFTYPE (START_ERR_CODE+60)
#define ERR_CAPTURE_TIMEOUT (START_ERR_CODE+61)
#define ERR_UNKNOWN (START_ERR_CODE+62)
#define ERR_CHAN_NOTOPEN (START_ERR_CODE+63)

typedef struct {
    int lastCmd;                    //last command sent to chan
    unsigned int chanTick;          //thread's tick (heartbeat)
    int putToSleep;                 //!=0 when thread put to sleep (fatal error)
    int lockPend;                   //1 when thread is waiting for mutex, 2 when owns mutex, 0 otherwise 
    int fvidStat;                   //video channel/buffer allocation status: 1 when success
    int noFreeBufJpg;               //no free buffers on the host
    int noFreeBufBmp;               //no free buffers on the host
} CHAN_STATUS;

typedef struct {
    CHAN_STATUS chanData[4];        //channel status data array
    unsigned int extErrAddr;        //address of extErr array
    unsigned int dspAddr;           //address of SYS_DATA in DSP mem space
    unsigned int fwVer;             //DSP firmware version: MAJOR[31:16] MINOR[15:0]
    unsigned int inPciRead;         //!=0 when PCI read (from host) in progress
    unsigned int inPciWrite;        //!=0 when PCI write (to host) in progress
    unsigned int hostLate;          //host has not cleared HDCR_CNANNEL_BUFFER in time (instance counter)
    unsigned int reserved[8];       //reserved
} SYS_STATUS;


#define SYS_BUF_PER_CHAN        4  /*buffers per channel*/
#define SYS_DSP_PER_BOARD       4  /*DSPs per board */
#define SYS_CHAN_PER_DSP        4  /*video channels per DSP*/

/*channels per board*/
#define SYS_CHAN_PER_BOARD  (SYS_CHAN_PER_DSP * SYS_DSP_PER_BOARD)
#define SYS_BMP_BUF_SIZE    (576*704)
#define SYS_JPG_BUF_SIZE    (80*1024)   /*jpeg buffer size */


#endif
